package cn.fastfun.service.impl;

import cn.fastfun.controller.dto.asset.BaseInfoDTO;
import cn.fastfun.controller.dto.asset.RentInfoDTO;
import cn.fastfun.controller.param.*;
import cn.fastfun.service.AppDeviceLogService;
import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.service.entity.AppDeviceLog;
import cn.fastfun.service.entity.AppImeiHistory;
import cn.fastfun.util.ObjectUtil;
import cn.fastfun.util.OkHttpCli;
import com.alibaba.fastjson.JSONObject;
import com.bridge.dto.ApiPageDTO;
import com.bridge.dto.QueryParam;
import com.bridge.dto.QueryParamExp;
import com.bridge.exception.ApiRuntimeException;
import com.bridge.service.JpaService;
import com.bridge.service.impl.JpaServiceImp;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.*;
import java.util.concurrent.Future;

@Slf4j
@Service
public class AppAssetProfitServiceImpl {
    @Autowired
    OkHttpCli okHttpCli;

    //基础信息
    @Async
    public Future<String> assetBatteryBasicInfo(AssetProfitParam param) {
        Map<String, String> mapHeaders = new HashMap<>();
//        mapHeaders.put("Cookie", cookie);
        JSONObject json = new JSONObject();
        json.put("sn", param.getSn());
        String response = okHttpCli.doPostTokenJson("http://192.168.0.131:8082/admin/v1/assetBatteryBasicInfo", json.toJSONString(), mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }

    // 时间线信息
    @Async
    public Future<String> getDateLine(AssetProfitParam param) {
        Map<String, String> mapHeaders = new HashMap<>();
//        mapHeaders.put("Cookie", cookie);
        JSONObject json = new JSONObject();
        json.put("sn", param.getSn());
        String response = okHttpCli.doPostTokenJson("http://192.168.0.131:8082/admin/v1/getDateLine", json.toJSONString(), mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }

    // 租赁信息列表
    @Async
    public Future<String> marketingInformation(AssetProfitParam param) {
        Map<String, String> mapHeaders = new HashMap<>();
//        mapHeaders.put("Cookie", cookie);
        JSONObject json = new JSONObject();
        json.put("sn", param.getSn());
        String response = okHttpCli.doPostTokenJson("http://192.168.0.131:8082/admin/v1/marketingInformation", json.toJSONString(), mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }

    // 资产基础信息
    public BaseInfoDTO getBaseInfo(AssetProfitParam param) throws Exception {
        BaseInfoDTO baseInfo = new BaseInfoDTO();
        List<String> response = new ArrayList<>();
        try {
            List<Future<String>> futures = new ArrayList<>();
            futures.add(assetBatteryBasicInfo(param));
            futures.add(getDateLine(param));
            //死循环，每隔2000ms执行一次，判断一下这三个异步调用的方法是否全都执行完了。
            while (true) {
                //使用Future的isDone()方法返回该方法是否执行完成
                if (futures.get(0).isDone() && futures.get(1).isDone()) {
                    //如果异步方法全部执行完，跳出循环
                    break;
                }
                //每隔200毫秒判断一次
                Thread.sleep(200);
            }
            for (Future<String> future:futures){
                String string = future.get();
                response.add(string);
            }
//            JSONObject jsonObject = JSONObject.parseObject(string);
        } catch (Exception e) {
            throw new Exception("" + e.getMessage());
        }

        baseInfo.setSn("");
        return baseInfo;
    }

    // 租赁信息
    public RentInfoDTO getRentInfo(String sn, String startTime, String endTime, Integer pageSize, Integer pageIndex){
        RentInfoDTO rentInfo = new RentInfoDTO();
        RentInfoDTO.AllInfo allInfo = new RentInfoDTO.AllInfo();
        allInfo.setAmount(new BigDecimal(10));
        allInfo.setTime("2021-10-21:00:00:00");
        allInfo.setDescription("测试");
        allInfo.setOrder("R123456");
        allInfo.setType("押金-收");
        rentInfo.getAllInfoList().add(allInfo);
        return rentInfo;
    }
}
